@@ -183,6 +183,86 @@ void probe_security_bprm_committed_creds(struct linux_binprm *bprm)
 	} while (0);
 }
 
+#if defined(CONFIG_ARM64)
+
+/**
+ * probe_security_hook_heads_on_arm - Find security_hook_heads on ARM64.
+ *
+ * @base: Address of security_bprm_committed_creds().
+ *
+ * Returns address of security_hook_heads.bprm_committed_creds on success,
+ * NULL otherwise.
+ */
+static void * __init probe_security_hook_heads_on_arm64(unsigned int *base)
+{
+	static unsigned int *ip4ret;
+	int i;
+	unsigned int *ip = (unsigned int *) base;
+
+	for (i = 0; i < 32; ip++, i++) {
+		unsigned long tmp;
+		unsigned long offset;
+		/*
+		 * Find
+		 *   adrp Xd, #imm21
+		 *   add Xd, Xn, #uimm12
+		 *   add Xd, Xn, #uimm12
+		 * sequence.
+		 * But don't mandate the last one in case it was optimized.
+		 */
+		if ((*ip & 0x98000000) != 0x90000000 ||
+		    (*(ip + 1) & 0xFFC00000) != 0x91000000)
+			continue;
+		tmp = ((unsigned long) ip) & ~0xFFFUL;
+		offset = (unsigned long) (((((*ip >> 5) & 0x007FFFF) << 2) |
+					   ((*ip >> 29) & 0x3))) << 12;
+		if (offset & 0x100000000UL)
+			offset |= 0xFFFFFFFF00000000UL;
+		tmp += offset;
+		offset = (*(ip + 1) >> 10) & 0xFFF;
+		tmp += offset;
+		if ((*(ip + 2) & 0xFFC00000) == 0x91000000) {
+			offset = (*(ip + 2) >> 10) & 0xFFF;
+			tmp += offset;
+		}
+		ip4ret = (unsigned int *) tmp;
+		return &ip4ret;
+	}
+	return NULL;
+}
+
+#endif
+
+#if defined(CONFIG_ARM)
+
+/**
+ * probe_security_hook_heads_on_arm - Find security_hook_heads on ARM.
+ *
+ * @base: Address of security_bprm_committed_creds().
+ *
+ * Returns address of security_hook_heads.bprm_committed_creds on success,
+ * NULL otherwise.
+ */
+static void * __init probe_security_hook_heads_on_arm(unsigned int *base)
+{
+	static unsigned int *ip4ret;
+	int i;
+	const unsigned long addr = (unsigned long) &probe_dummy_security_hook_heads;
+	const unsigned long offset = (unsigned long) &probe_dummy_security_hook_heads.bprm_committed_creds - addr;
+	unsigned int *ip = (unsigned int *) probe_security_bprm_committed_creds;
+	for (i = 0; i < 32; ip++, i++) {
+		if (*(ip + 2 + ((*ip & 0xFFF) >> 2)) != addr)
+			continue;
+		ip = base + i;
+		ip4ret = (unsigned int *) (*(ip + 2 + ((*ip & 0xFFF) >> 2)));
+		ip4ret += offset >> 2;
+		return &ip4ret;
+	}
+	return NULL;
+}
+
+#endif
+
 #elif LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
 
 /*
@@ -311,6 +391,14 @@ static void * __init probe_find_variable(void *function, unsigned long addr,
 		base = __symbol_get(symbol);
 	if (!base)
 		return NULL;
+#if defined(CONFIG_ARM64) && defined(LSM_HOOK_INIT)
+	if (function == probe_security_bprm_committed_creds)
+		return probe_security_hook_heads_on_arm64((unsigned int *) base);
+#endif
+#if defined(CONFIG_ARM) && defined(LSM_HOOK_INIT)
+	if (function == probe_security_bprm_committed_creds)
+		return probe_security_hook_heads_on_arm((unsigned int *) base);
+#endif
 #if defined(CONFIG_ARM) && LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24) && !defined(LSM_HOOK_INIT)
 	if (function == probe_security_file_alloc)
 		return probe_security_ops_on_arm((unsigned int *) base);
